@@ -39,13 +39,15 @@ impl AppState {
 
     /// Save the content of the edited cell if any, then save the whole closet
     fn save(&mut self, reopen_if_open: bool) -> Result<(), SafeClosetError> {
-        self.close_drawer_input(false);
+        time!(self.close_drawer_input(false));
         let drawer_state = std::mem::take(&mut self.drawer_state);
         if let DrawerState::DrawerEdit(des) = drawer_state {
             if reopen_if_open {
-                self.drawer_state = DrawerState::DrawerEdit(des.close_and_reopen(&mut self.closet)?);
+                self.drawer_state = DrawerState::DrawerEdit(
+                    time!(des.close_and_reopen(&mut self.closet)?)
+                );
             } else {
-                self.closet.close_drawer(des.drawer)?;
+                time!(self.closet.close_drawer(des.drawer)?);
             }
         }
         self.closet.save()?;
@@ -77,9 +79,50 @@ impl AppState {
             return Ok(CmdResult::Quit);
         }
 
-        // -- pending removal
+        if key == CONTROL_UP { // moving the selected line up
+            if let DrawerEdit(des) = &mut self.drawer_state {
+                des.close_input(false);
+                let len = des.drawer.entries.len();
+                match des.focus {
+                    NameSelected { line } => {
+                        let new_line = (line + len - 1) % len;
+                        des.drawer.entries.swap(line, new_line);
+                        des.focus = NameSelected { line: new_line };
+                    }
+                    ValueSelected { line } => {
+                        let new_line = (line + len - 1) % len;
+                        des.drawer.entries.swap(line, new_line);
+                        des.focus = ValueSelected { line: new_line };
+                    }
+                    _ => {}
+                }
+            }
+            return Ok(CmdResult::Stay);
+        }
+        if key == CONTROL_DOWN { // moving the selected line down
+            if let DrawerEdit(des) = &mut self.drawer_state {
+                des.close_input(false);
+                let len = des.drawer.entries.len();
+                match des.focus {
+                    NameSelected { line } => {
+                        let new_line = (line + 1) % len;
+                        des.drawer.entries.swap(line, new_line);
+                        des.focus = NameSelected { line: new_line };
+                    }
+                    ValueSelected { line } => {
+                        let new_line = (line + 1) % len;
+                        des.drawer.entries.swap(line, new_line);
+                        des.focus = ValueSelected { line: new_line };
+                    }
+                    _ => {}
+                }
+            }
+            return Ok(CmdResult::Stay);
+        }
+
 
         if let DrawerEdit(des) = &mut self.drawer_state {
+            // -- pending removal
             if let PendingRemoval { line } = &des.focus {
                 let line = *line;
                 if let Some(idx) = des.listed_entry_idx(line) {
@@ -97,6 +140,7 @@ impl AppState {
                         des.focus = NameSelected { line };
                     }
                 }
+                return Ok(CmdResult::Stay);
             }
         }
 
@@ -119,16 +163,16 @@ impl AppState {
             self.close_drawer_input(false); // if there's an entry input
             if let DrawerCreation(PasswordInputState { input }) = &mut self.drawer_state {
                 let pwd = input.get_content();
-                let open_drawer = self
-                    .closet
-                    .create_drawer(&pwd)
+                let creation = time!(self.closet.create_drawer(&pwd));
+                let open_drawer = creation
                     .map_err(|e| e.into())
                     .and_then(|_| {
-                        self.closet.open_drawer(&pwd).ok_or_else(|| {
-                            SafeClosetError::Internal(
-                                "unexpected failure opening drawer".to_string(),
-                            )
-                        })
+                        time!(self.closet.open_drawer(&pwd))
+                            .ok_or_else(|| {
+                                SafeClosetError::Internal(
+                                    "unexpected failure opening drawer".to_string(),
+                                )
+                            })
                     });
                 match open_drawer {
                     Ok(open_drawer) => {
@@ -169,8 +213,8 @@ impl AppState {
             return Ok(CmdResult::Stay);
         }
 
-        if let DrawerEdit(des) = &mut self.drawer_state {
-            if key == TAB {
+        if key == TAB {
+            if let DrawerEdit(des) = &mut self.drawer_state {
                 if matches!(des.focus, NoneSelected) {
                     // we remove any search
                     des.search.clear();
@@ -204,6 +248,24 @@ impl AppState {
                 }
                 return Ok(CmdResult::Stay);
             }
+        }
+
+        // --- input
+
+        if let Some(input) = self.drawer_state.input() {
+            if input.apply_key_event(key) {
+                if let DrawerEdit(des) = &mut self.drawer_state {
+                    if des.focus.is_search() {
+                        des.search.update(&des.drawer);
+                    }
+                }
+                return Ok(CmdResult::Stay);
+            }
+        }
+
+        // --- navigation among entries
+
+        if let DrawerEdit(des) = &mut self.drawer_state {
             if key == HOME {
                 des.apply_scroll_command(ScrollCommand::Top);
                 return Ok(CmdResult::Stay);
@@ -222,21 +284,6 @@ impl AppState {
             }
         }
 
-        // --- input
-
-        if let Some(input) = self.drawer_state.input() {
-            if input.apply_keycode_event(key.code) {
-                if let DrawerEdit(des) = &mut self.drawer_state {
-                    if des.focus.is_search() {
-                        des.search.update(&des.drawer);
-                    }
-                }
-                return Ok(CmdResult::Stay);
-            }
-        }
-
-        // --- navigation among entries
-
         if key == INSERT || as_letter(key) == Some('i') {
             if let DrawerEdit(des) = &mut self.drawer_state {
                 if let NameSelected { line } = &des.focus {
